package com.arms.api.issue.type.strategy;

import com.arms.api.issue.type.dto.IssueTypeDTO;
import com.arms.api.issue.type.vo.IssueTypeVO;
import com.arms.api.serverinfo.model.서버정보_데이터;
import com.arms.api.util.alm.지라API_정보;
import com.arms.api.util.alm.지라유틸;
import com.arms.api.util.errors.ErrorLogUtil;
import lombok.AllArgsConstructor;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.stereotype.Component;
import org.springframework.web.reactive.function.client.WebClient;

import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

@Component
@AllArgsConstructor
public class CloudJiraIssueTypeStrategy implements IssueTypeStrategy {

    private final 지라유틸 지라유틸;
    private final 지라API_정보 지라API_정보;

    @Override
    public List<IssueTypeVO> getIssueTypeList(서버정보_데이터 서버정보) {

        String endpoint = 지라API_정보.getEndpoint().getIssue().getIssuetype();
        WebClient webClient = 지라유틸.클라우드_통신기_생성(서버정보.getUri(), 서버정보.getUserId(), 서버정보.getPasswordOrToken());

        try {
            List<IssueTypeDTO> issueTypeList = 지라유틸.get(webClient, endpoint,
                    new ParameterizedTypeReference<List<IssueTypeDTO>>() {}).block();

            return Optional.ofNullable(issueTypeList)
                    .orElse(Collections.emptyList())
                    .stream()
                    .filter(Objects::nonNull)
                    .map(this::convertIssueTypeVO)
                    .collect(Collectors.toList());
        }
        catch (Exception e) {
            String errorMessage = ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                    String.format("%s[%s] :: getIssueTypeList Error", 서버정보.getType(), 서버정보.getUri()));
            throw new IllegalArgumentException(errorMessage);
        }
    }

    @Override
    public List<IssueTypeVO> getIssueTypeListByProject(서버정보_데이터 서버정보, String projectId) {

        String endpoint = 지라API_정보.getEndpoint().getIssue().getIssuetype() + "/project?projectId=" + projectId;
        WebClient webClient = 지라유틸.클라우드_통신기_생성(서버정보.getUri(), 서버정보.getUserId(), 서버정보.getPasswordOrToken());

        try {
            List<IssueTypeDTO> issueTypeList = 지라유틸.get(webClient, endpoint,
                    new ParameterizedTypeReference<List<IssueTypeDTO>>() {}).block();

            return Optional.ofNullable(issueTypeList)
                    .orElse(Collections.emptyList())
                    .stream()
                    .filter(Objects::nonNull)
                    .map(this::convertIssueTypeVO)
                    .collect(Collectors.toList());
        }
        catch (Exception e) {
            String errorMessage = ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                    String.format("%s[%s] :: getIssueTypeListByProject Error :: projectId[%s]",
                                서버정보.getType(), 서버정보.getUri(), projectId));
            throw new IllegalArgumentException(errorMessage);
        }
    }

    private IssueTypeVO convertIssueTypeVO(IssueTypeDTO issueTypeDTO) {
        return IssueTypeVO.builder()
                .self(issueTypeDTO.getSelf())
                .id(issueTypeDTO.getId())
                .description(issueTypeDTO.getDescription())
                .name(issueTypeDTO.getName())
                .subtask(issueTypeDTO.getSubtask())
                .untranslatedName(issueTypeDTO.getUntranslatedName())
                .hierarchyLevel(issueTypeDTO.getHierarchyLevel()).build();
    }
}